@@ -1,6 +1,6 @@
 # manager.py
 #
-# Copyright 2022 brombinmirko <send@mirko.pm>
+# Copyright 2025 mirkobrombin <brombin94@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,10 +23,10 @@ import shutil
 import subprocess
 import time
 import uuid
-from threading import Event
 from datetime import datetime
 from gettext import gettext as _
 from glob import glob
+from threading import Event
 from typing import Any, Callable, Dict, List, Optional, Tuple
 
 import pathvalidate
@@ -44,21 +44,26 @@ from bottles.backend.managers.epicgamesstore import EpicGamesStoreManager
 from bottles.backend.managers.importer import ImportManager
 from bottles.backend.managers.installer import InstallerManager
 from bottles.backend.managers.library import LibraryManager
+from bottles.backend.managers.playtime import ProcessSessionTracker
+from bottles.backend.managers.registry_rule import RegistryRuleManager
 from bottles.backend.managers.repository import RepositoryManager
 from bottles.backend.managers.steam import SteamManager
 from bottles.backend.managers.template import TemplateManager
 from bottles.backend.managers.ubisoftconnect import UbisoftConnectManager
 from bottles.backend.managers.versioning import VersioningManager
 from bottles.backend.models.config import BottleConfig
+from bottles.backend.models.process import (
+    ProcessFinishedPayload,
+    ProcessStartedPayload,
+)
 from bottles.backend.models.result import Result
 from bottles.backend.models.samples import Samples
-from bottles.backend.state import SignalManager, Signals, Events, EventManager
+from bottles.backend.state import EventManager, Events, SignalManager, Signals
 from bottles.backend.utils import yaml
 from bottles.backend.utils.connection import ConnectionUtils
 from bottles.backend.utils.file import FileUtils
 from bottles.backend.utils.generic import sort_by_version
-from bottles.backend.utils.gpu import GPUUtils
-from bottles.backend.utils.gpu import GPUVendors
+from bottles.backend.utils.gpu import GPUUtils, GPUVendors
 from bottles.backend.utils.gsettings_stub import GSettingsStub
 from bottles.backend.utils.lnk import LnkUtils
 from bottles.backend.utils.manager import ManagerUtils
@@ -103,6 +108,7 @@ class Manager(metaclass=Singleton):
     supported_latencyflex = {}
     supported_dependencies = {}
     supported_installers = {}
+    _playtime_signals_connected: bool = False
 
     def __init__(
         self,
@@ -154,6 +160,25 @@ class Manager(metaclass=Singleton):
         self.steam_manager = SteamManager()
         times["SteamManager"] = time.time()
 
+        # Initialize playtime tracker
+        self._initialize_playtime_tracker()
+        times["PlaytimeTracker"] = time.time()
+
+        # React to runtime changes in playtime preference when available
+        if hasattr(self.settings, "connect"):
+            try:
+                self.settings.connect(
+                    "changed::playtime-enabled", self._on_playtime_enabled_changed
+                )
+            except Exception:
+                pass
+
+        # Subscribe to playtime signals (connect once per process)
+        if not Manager._playtime_signals_connected:
+            SignalManager.connect(Signals.ProgramStarted, self._on_program_started)
+            SignalManager.connect(Signals.ProgramFinished, self._on_program_finished)
+            Manager._playtime_signals_connected = True
+
         if not self.is_cli:
             times.update(self.checks(install_latest=False, first_run=True).data)
         else:
@@ -288,6 +313,146 @@ class Manager(metaclass=Singleton):
 
         return rv
 
+    def __del__(self):
+        # best-effort shutdown of playtime tracker
+        try:
+            if hasattr(self, "playtime_tracker") and self.playtime_tracker:
+                self.playtime_tracker.shutdown()
+        except Exception:
+            pass
+
+    def _initialize_playtime_tracker(self) -> None:
+        playtime_enabled = self.settings.get_boolean("playtime-enabled")
+        playtime_interval = self.settings.get_int("playtime-heartbeat-interval")
+        tracker = ProcessSessionTracker(
+            enabled=playtime_enabled,
+            heartbeat_interval=playtime_interval if playtime_interval > 0 else 60,
+        )
+        tracker.recover_open_sessions()
+        self.playtime_tracker = tracker
+
+    def _on_playtime_enabled_changed(self, _settings, _key) -> None:
+        enabled = self.settings.get_boolean("playtime-enabled")
+        if not enabled:
+            if (
+                getattr(self, "playtime_tracker", None)
+                and self.playtime_tracker.enabled
+            ):
+                self._launch_to_session.clear()
+                self.playtime_tracker.disable_tracking()
+            return
+
+        if getattr(self, "playtime_tracker", None) and self.playtime_tracker.enabled:
+            return
+
+        self._initialize_playtime_tracker()
+
+    # Playtime signal handlers
+    _launch_to_session: Dict[str, int] = {}
+
+    # Public Playtime API (wrap tracker with Result)
+    def playtime_start(
+        self,
+        *,
+        bottle_id: str,
+        bottle_name: str,
+        bottle_path: str,
+        program_name: str,
+        program_path: str,
+    ) -> Result[int]:
+        try:
+            sid = self.playtime_tracker.start_session(
+                bottle_id=bottle_id,
+                bottle_name=bottle_name,
+                bottle_path=bottle_path,
+                program_name=program_name,
+                program_path=program_path,
+            )
+            return Result(True, data=sid)
+        except Exception as e:
+            logging.exception(e)
+            return Result(False, message=str(e))
+
+    def playtime_finish(
+        self,
+        session_id: int,
+        *,
+        status: str = "success",
+        ended_at: Optional[int] = None,
+    ) -> Result[None]:
+        try:
+            if status == "success":
+                self.playtime_tracker.mark_exit(
+                    session_id, status="success", ended_at=ended_at
+                )
+            else:
+                self.playtime_tracker.mark_failure(session_id, status=status)
+            return Result(True)
+        except Exception as e:
+            logging.exception(e)
+            return Result(False, message=str(e))
+
+    def _on_program_started(self, data: Optional[Result] = None) -> None:
+        try:
+            if not data or not data.data:
+                return
+            payload: ProcessStartedPayload = data.data  # type: ignore
+            logging.debug(
+                f"Playtime signal: started launch_id={payload.launch_id} bottle={payload.bottle_name} program={payload.program_name}"
+            )
+            res = self.playtime_start(
+                bottle_id=payload.bottle_id,
+                bottle_name=payload.bottle_name,
+                bottle_path=payload.bottle_path,
+                program_name=payload.program_name,
+                program_path=payload.program_path,
+            )
+            if not res.ok:
+                return
+            sid = int(res.data or -1)
+            self._launch_to_session[payload.launch_id] = sid
+
+            config = self._get_payload_config(payload)
+            if config:
+                RegistryRuleManager.apply_rules(config, trigger="start_program")
+        except Exception:
+            pass
+
+    def _on_program_finished(self, data: Optional[Result] = None) -> None:
+        try:
+            if not data or not data.data:
+                return
+            payload: ProcessFinishedPayload = data.data  # type: ignore
+            sid = self._launch_to_session.pop(payload.launch_id, -1)
+            if sid and sid > 0:
+                status = payload.status
+                ended_at = int(payload.ended_at or time.time())
+                logging.debug(
+                    f"Playtime signal: finished launch_id={payload.launch_id} status={status} sid={sid}"
+                )
+                self.playtime_finish(sid, status=status, ended_at=ended_at)
+
+            config = self._get_payload_config(payload)
+            if config:
+                RegistryRuleManager.apply_rules(config, trigger="stop_program")
+        except Exception:
+            pass
+
+    def _get_payload_config(self, payload) -> Optional[BottleConfig]:
+        config = self.local_bottles.get(payload.bottle_name)
+        if isinstance(config, BottleConfig):
+            return config
+
+        try:
+            config_path = os.path.join(payload.bottle_path, "bottle.yml")
+        except Exception:
+            return None
+
+        loaded = BottleConfig.load(config_path)
+        if loaded.status:
+            return loaded.data
+        return None
+
     def __clear_temp(self, force: bool = False):
         """Clears the temp directory if user setting allows it. Use the force
         parameter to force clearing the directory.
@@ -300,6 +465,87 @@ class Manager(metaclass=Singleton):
             except FileNotFoundError:
                 self.check_app_dirs()
 
+    def get_cache_details(self) -> dict:
+        self.check_app_dirs()
+        file_utils = FileUtils()
+
+        temp_size_bytes = file_utils.get_path_size(Paths.temp, human=False)
+        templates = []
+        templates_size_bytes = 0
+
+        for template in TemplateManager.get_templates():
+            template_uuid = template.get("uuid", "")
+            template_path = os.path.join(Paths.templates, template_uuid)
+
+            size_bytes = file_utils.get_path_size(template_path, human=False)
+            templates_size_bytes += size_bytes
+
+            templates.append(
+                {
+                    "uuid": template_uuid,
+                    "env": template.get("env", ""),
+                    "created": template.get("created", ""),
+                    "size": file_utils.get_human_size(size_bytes),
+                    "size_bytes": size_bytes,
+                }
+            )
+
+        total_size_bytes = temp_size_bytes + templates_size_bytes
+
+        return {
+            "temp": {
+                "path": Paths.temp,
+                "size": file_utils.get_human_size(temp_size_bytes),
+                "size_bytes": temp_size_bytes,
+            },
+            "templates": templates,
+            "templates_size": file_utils.get_human_size(templates_size_bytes),
+            "templates_size_bytes": templates_size_bytes,
+            "total_size": file_utils.get_human_size(total_size_bytes),
+            "total_size_bytes": total_size_bytes,
+        }
+
+    def clear_temp_cache(self) -> Result[None]:
+        try:
+            self.__clear_temp(force=True)
+        except Exception as ex:
+            logging.error(f"Failed to clear temp cache: {ex}")
+            return Result(False, message=str(ex))
+
+        return Result(True)
+
+    def clear_template_cache(self, template_uuid: str) -> Result[None]:
+        self.check_app_dirs()
+        try:
+            TemplateManager.delete_template(template_uuid)
+        except Exception as ex:
+            logging.error(f"Failed to clear template cache: {ex}")
+            return Result(False, message=str(ex))
+
+        return Result(True)
+
+    def clear_templates_cache(self) -> Result[None]:
+        self.check_app_dirs()
+        try:
+            for template in TemplateManager.get_templates():
+                TemplateManager.delete_template(template.get("uuid", ""))
+        except Exception as ex:
+            logging.error(f"Failed to clear templates cache: {ex}")
+            return Result(False, message=str(ex))
+
+        return Result(True)
+
+    def clear_all_caches(self) -> Result[None]:
+        temp_result = self.clear_temp_cache()
+        if not temp_result.ok:
+            return temp_result
+
+        templates_result = self.clear_templates_cache()
+        if not templates_result.ok:
+            return templates_result
+
+        return Result(True)
+
     def update_bottles(self, silent: bool = False):
         """Checks for new bottles and update the list view."""
         self.check_bottles(silent)
@@ -548,32 +794,77 @@ class Manager(metaclass=Singleton):
                     return True
         return False
 
-    def check_winebridge(
-        self, install_latest: bool = True, update: bool = False
-    ) -> bool:
+    def __winebridge_status(self) -> tuple[Optional[str], Optional[str], bool]:
+        def _is_newer(candidate: str, current: str) -> bool:
+            versions = [candidate, current]
+            try:
+                sorted_versions = sort_by_version(list(versions))
+            except ValueError:
+                sorted_versions = sorted(versions, reverse=True)
+            return sorted_versions[0] == candidate and candidate != current
+
         self.winebridge_available = []
         winebridge = os.listdir(Paths.winebridge)
+        latest_supported = None
 
-        if len(winebridge) == 0 or update:
-            if install_latest and self.utils_conn.check_connection():
-                logging.warning("No WineBridge found.")
-                try:
-                    version = next(iter(self.supported_winebridge))
-                    self.component_manager.install("winebridge", version)
-                    self.winebridge_available = [version]
-                    return True
-                except StopIteration:
-                    return False
-            return False
+        if self.supported_winebridge:
+            try:
+                latest_supported = sort_by_version(
+                    list(self.supported_winebridge.keys())
+                )[0]
+            except ValueError:
+                latest_supported = sorted(
+                    list(self.supported_winebridge.keys()), reverse=True
+                )[0]
 
         version_file = os.path.join(Paths.winebridge, "VERSION")
+        installed_identifier = None
         if os.path.exists(version_file):
             with open(version_file, "r") as f:
                 version = f.read().strip()
                 if version:
-                    self.winebridge_available = [f"winebridge-{version}"]
-                    return True
-        return False
+                    installed_identifier = f"winebridge-{version}"
+                    self.winebridge_available = [installed_identifier]
+
+        missing_installation = len(winebridge) == 0 or not installed_identifier
+        needs_latest = False
+        if latest_supported:
+            needs_latest = (
+                missing_installation
+                or _is_newer(latest_supported, installed_identifier)
+            )
+
+        return latest_supported, installed_identifier, needs_latest
+
+    def winebridge_update_status(self) -> dict:
+        latest_supported, installed_identifier, needs_latest = self.__winebridge_status()
+        return {
+            "latest_supported": latest_supported,
+            "installed_identifier": installed_identifier,
+            "needs_latest": needs_latest,
+            "missing": not installed_identifier,
+        }
+
+    def check_winebridge(
+        self, install_latest: bool = True, update: bool = False
+    ) -> bool:
+        latest_supported, installed_identifier, needs_latest = self.__winebridge_status()
+
+        can_install = install_latest or update
+        if can_install and needs_latest and latest_supported:
+            if not self.utils_conn.check_connection():
+                return False
+            logging.warning("WineBridge installation/update required.")
+            res = self.component_manager.install("winebridge", latest_supported)
+            if res.ok:
+                self.winebridge_available = [latest_supported]
+                return True
+            return False
+
+        if needs_latest and not can_install:
+            return False
+
+        return bool(self.winebridge_available)
 
     def check_dxvk(self, install_latest: bool = True) -> bool:
         res = self.__check_component("dxvk", install_latest)
@@ -798,7 +1089,9 @@ class Manager(metaclass=Singleton):
                     "path": _program.get("path"),
                     "icon": "com.usebottles.bottles-program",
                     "pre_script": _program.get("pre_script"),
+                    "pre_script_args": _program.get("pre_script_args"),
                     "post_script": _program.get("post_script"),
+                    "post_script_args": _program.get("post_script_args"),
                     "folder": _program.get("folder", program_folder),
                     "dxvk": _program.get("dxvk"),
                     "vkd3d": _program.get("vkd3d"),
@@ -807,6 +1100,7 @@ class Manager(metaclass=Singleton):
                     "gamescope": _program.get("gamescope"),
                     "pulseaudio_latency": _program.get("pulseaudio_latency"),
                     "virtual_desktop": _program.get("virtual_desktop"),
+                    "winebridge": _program.get("winebridge"),
                     "removed": _program.get("removed"),
                     "id": _program.get("id"),
                 }
@@ -1093,8 +1387,35 @@ class Manager(metaclass=Singleton):
         if config.Environment == "Steam":
             self.steam_manager.update_bottle(config)
 
+        component_keys = {
+            "Runner",
+            "DXVK",
+            "VKD3D",
+            "NVAPI",
+            "LatencyFleX",
+            "LatencyFleX_Activated",
+        }
+
+        if key in component_keys or scope in component_keys:
+            RegistryRuleManager.apply_rules(config, trigger="components")
+
         return Result(status=True, data={"config": config})
 
+    def apply_audio_driver(self, driver: str) -> Result[None]:
+        """Apply the configured audio driver override to every bottle."""
+
+        logging.info(f"Applying audio driver '{driver}' to local bottles…")
+        try:
+            for config in self.local_bottles.values():
+                if not isinstance(config, BottleConfig):
+                    continue
+                RegKeys(config).set_audio_driver(driver)
+        except ValueError as exc:
+            logging.error(str(exc))
+            return Result(False, message=str(exc))
+
+        return Result(True)
+
     def create_bottle_from_config(self, config: BottleConfig) -> bool:
         """Create a bottle from a config object."""
         logging.info(f"Creating new {config.Name} bottle from config…")
@@ -1419,6 +1740,10 @@ class Manager(metaclass=Singleton):
         config.Update_Date = str(datetime.now())
         if versioning:
             config.Versioning = True
+        config.Limit_System_Environment = True
+        config.Inherited_Environment_Variables = (
+            Samples.default_inherited_environment.copy()
+        )
 
         cleanup_config = config
 
@@ -1527,6 +1852,16 @@ class Manager(metaclass=Singleton):
             logging.info("Enabling font smoothing…")
             log_update(_("Enabling font smoothing…"))
             rk.apply_font_smoothing()
+
+            audio_driver = self.settings.get_string("audio-driver")
+            if audio_driver not in ("", "default"):
+                logging.info("Configuring audio driver…")
+                log_update(_("Configuring audio driver…"))
+                try:
+                    rk.set_audio_driver(audio_driver)
+                except ValueError as exc:
+                    logging.warning(str(exc))
+
             wineboot.update()
 
             FileUtils.wait_for_files(reg_files)
