@@ -1,6 +1,6 @@
 # bottle_preferences.py
 #
-# Copyright 2022 brombinmirko <send@mirko.pm>
+# Copyright 2025 mirkobrombin <brombin94@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,16 +20,15 @@ import os
 import re
 from gettext import gettext as _
 
-from gi.repository import Gtk, Adw
+from gi.repository import Adw, Gdk, Gtk
 
 from bottles.backend.globals import (
     gamemode_available,
-    vkbasalt_available,
+    gamescope_available,
     mangohud_available,
     obs_vkc_available,
+    vkbasalt_available,
     vmtouch_available,
-    gamescope_available,
-    base_version,
 )
 from bottles.backend.logger import Logger
 from bottles.backend.managers.library import LibraryManager
@@ -38,6 +37,7 @@ from bottles.backend.models.config import BottleConfig
 from bottles.backend.models.enum import Arch
 from bottles.backend.models.result import Result
 from bottles.backend.runner import Runner
+from bottles.backend.utils.display import DisplayUtils
 from bottles.backend.utils.gpu import GPUUtils, GPUVendors
 from bottles.backend.utils.manager import ManagerUtils
 from bottles.backend.utils.threading import RunAsync
@@ -77,6 +77,13 @@ class PreferencesView(Adw.PreferencesPage):
     row_nvapi = Gtk.Template.Child()
     row_discrete = Gtk.Template.Child()
     row_vkbasalt = Gtk.Template.Child()
+    row_gamescope = Gtk.Template.Child()
+    row_mangohud = Gtk.Template.Child()
+    row_gamemode = Gtk.Template.Child()
+    row_vmtouch = Gtk.Template.Child()
+    row_obsvkc = Gtk.Template.Child()
+    row_wayland = Gtk.Template.Child()
+    row_winebridge = Gtk.Template.Child()
     row_manage_display = Gtk.Template.Child()
     row_runtime = Gtk.Template.Child()
     row_steam_runtime = Gtk.Template.Child()
@@ -90,6 +97,8 @@ class PreferencesView(Adw.PreferencesPage):
     switch_mangohud = Gtk.Template.Child()
     switch_obsvkc = Gtk.Template.Child()
     switch_vkbasalt = Gtk.Template.Child()
+    switch_wayland = Gtk.Template.Child()
+    switch_winebridge = Gtk.Template.Child()
     switch_fsr = Gtk.Template.Child()
     switch_nvapi = Gtk.Template.Child()
     switch_gamemode = Gtk.Template.Child()
@@ -143,79 +152,76 @@ class PreferencesView(Adw.PreferencesPage):
             return
 
         _not_available = _("This feature is unavailable on your system.")
-        _flatpak_not_available = _(
-            "{} To add this feature, please run flatpak install"
-        ).format(_not_available)
-        _gamescope_pkg_name = "org.freedesktop.Platform.VulkanLayer.gamescope"
-        _vkbasalt_pkg_name = "org.freedesktop.Platform.VulkanLayer.vkBasalt"
-        _mangohud_pkg_name = "org.freedesktop.Platform.VulkanLayer.MangoHud"
-        _obsvkc_pkg_name = "com.obsproject.Studio.Plugin.OBSVkCapture"
-        _flatpak_pkg_name = {
-            "gamescope": (
-                f"{_gamescope_pkg_name}//{base_version}"
-                if base_version
-                else _gamescope_pkg_name
-            ),
-            "vkbasalt": (
-                f"{_vkbasalt_pkg_name}//{base_version}"
-                if base_version
-                else _vkbasalt_pkg_name
-            ),
-            "mangohud": (
-                f"{_mangohud_pkg_name}//{base_version}"
-                if base_version
-                else _mangohud_pkg_name
-            ),
-            "obsvkc": _obsvkc_pkg_name,
+        _flatpak_not_available = _("{} To add this feature, please run").format(
+            _not_available
+        )
+        self._install_commands = {
+            "gamescope": "flatpak install flathub org.freedesktop.Platform.VulkanLayer.gamescope",
+            "vkbasalt": "flatpak install flathub org.freedesktop.Platform.VulkanLayer.vkBasalt",
+            "mangohud": "flatpak install flathub org.freedesktop.Platform.VulkanLayer.MangoHud",
+            "obsvkc": "flatpak install flathub com.obsproject.Studio.Plugin.OBSVkCapture",
         }
 
+        is_flatpak = "FLATPAK_ID" in os.environ
+
         if not gamemode_available:
             self.switch_gamemode.set_tooltip_text(_not_available)
+            self.__add_unavailable_indicator(self.row_gamemode, None)
 
         if not gamescope_available:
-            if "FLATPAK_ID" in os.environ:
-                _gamescope_not_available = (
-                    f"{_flatpak_not_available} {_flatpak_pkg_name['gamescope']}"
-                )
-                self.switch_gamescope.set_tooltip_text(_gamescope_not_available)
-                self.btn_manage_gamescope.set_tooltip_text(_gamescope_not_available)
-            else:
-                self.switch_gamescope.set_tooltip_text(_not_available)
-                self.btn_manage_gamescope.set_tooltip_text(_not_available)
+            _gamescope_command = self._install_commands.get("gamescope")
+            _gamescope_not_available = (
+                f"{_flatpak_not_available} {_gamescope_command}"
+                if is_flatpak
+                else _not_available
+            )
+            self.switch_gamescope.set_tooltip_text(_gamescope_not_available)
+            self.btn_manage_gamescope.set_tooltip_text(_gamescope_not_available)
+            self.__add_unavailable_indicator(
+                self.row_gamescope, _gamescope_command if is_flatpak else None
+            )
 
         if not vkbasalt_available:
-            if "FLATPAK_ID" in os.environ:
-                _vkbasalt_not_available = (
-                    f"{_flatpak_not_available} {_flatpak_pkg_name['vkbasalt']}"
-                )
-                self.switch_vkbasalt.set_tooltip_text(_vkbasalt_not_available)
-                self.btn_manage_vkbasalt.set_tooltip_text(_vkbasalt_not_available)
-            else:
-                self.switch_vkbasalt.set_tooltip_text(_not_available)
-                self.btn_manage_vkbasalt.set_tooltip_text(_not_available)
+            _vkbasalt_command = self._install_commands.get("vkbasalt")
+            _vkbasalt_not_available = (
+                f"{_flatpak_not_available} {_vkbasalt_command}"
+                if is_flatpak
+                else _not_available
+            )
+            self.switch_vkbasalt.set_tooltip_text(_vkbasalt_not_available)
+            self.btn_manage_vkbasalt.set_tooltip_text(_vkbasalt_not_available)
+            self.__add_unavailable_indicator(
+                self.row_vkbasalt, _vkbasalt_command if is_flatpak else None
+            )
 
         if not mangohud_available:
-            if "FLATPAK_ID" in os.environ:
-                _mangohud_not_available = (
-                    f"{_flatpak_not_available} {_flatpak_pkg_name['mangohud']}"
-                )
-                self.switch_mangohud.set_tooltip_text(_mangohud_not_available)
-                self.btn_manage_mangohud.set_tooltip_text(_mangohud_not_available)
-            else:
-                self.switch_mangohud.set_tooltip_text(_not_available)
-                self.btn_manage_mangohud.set_tooltip_text(_not_available)
+            _mangohud_command = self._install_commands.get("mangohud")
+            _mangohud_not_available = (
+                f"{_flatpak_not_available} {_mangohud_command}"
+                if is_flatpak
+                else _not_available
+            )
+            self.switch_mangohud.set_tooltip_text(_mangohud_not_available)
+            self.btn_manage_mangohud.set_tooltip_text(_mangohud_not_available)
+            self.__add_unavailable_indicator(
+                self.row_mangohud, _mangohud_command if is_flatpak else None
+            )
 
         if not obs_vkc_available:
-            if "FLATPAK_ID" in os.environ:
-                _obsvkc_not_available = (
-                    f"{_flatpak_not_available} {_flatpak_pkg_name['obsvkc']}"
-                )
-                self.switch_obsvkc.set_tooltip_text(_obsvkc_not_available)
-            else:
-                self.switch_obsvkc.set_tooltip_text(_not_available)
+            _obsvkc_command = self._install_commands.get("obsvkc")
+            _obsvkc_not_available = (
+                f"{_flatpak_not_available} {_obsvkc_command}"
+                if is_flatpak
+                else _not_available
+            )
+            self.switch_obsvkc.set_tooltip_text(_obsvkc_not_available)
+            self.__add_unavailable_indicator(
+                self.row_obsvkc, _obsvkc_command if is_flatpak else None
+            )
 
         if not vmtouch_available:
             self.switch_vmtouch.set_tooltip_text(_not_available)
+            self.__add_unavailable_indicator(self.row_vmtouch, None)
 
         # region signals
         self.row_manage_display.connect("activated", self.__show_display_settings)
@@ -250,6 +256,10 @@ class PreferencesView(Adw.PreferencesPage):
         self.switch_mangohud.connect("state-set", self.__toggle_feature_cb, "mangohud")
         self.switch_obsvkc.connect("state-set", self.__toggle_feature_cb, "obsvkc")
         self.switch_vkbasalt.connect("state-set", self.__toggle_feature_cb, "vkbasalt")
+        self.switch_wayland.connect("state-set", self.__toggle_wayland)
+        self.switch_winebridge.connect(
+            "state-set", self.__toggle_feature_cb, "winebridge"
+        )
         self.switch_fsr.connect("state-set", self.__toggle_feature_cb, "fsr")
         self.switch_nvapi.connect("state-set", self.__toggle_nvapi)
         self.switch_gamemode.connect("state-set", self.__toggle_feature_cb, "gamemode")
@@ -304,6 +314,77 @@ class PreferencesView(Adw.PreferencesPage):
         self.switch_obsvkc.set_sensitive(obs_vkc_available)
         self.switch_vmtouch.set_sensitive(vmtouch_available)
 
+        is_wayland_session = DisplayUtils.display_server_type() == "wayland"
+        self.switch_wayland.set_sensitive(is_wayland_session)
+
+    def __create_unavailable_popover(self, command: str | None) -> Gtk.Popover:
+        popover = Gtk.Popover()
+        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6)
+        box.set_margin_top(6)
+        box.set_margin_bottom(6)
+        box.set_margin_start(6)
+        box.set_margin_end(6)
+
+        unavailable_label = Gtk.Label(
+            label=_("This feature is unavailable on your system."),
+            xalign=0,
+            wrap=True,
+        )
+        box.append(unavailable_label)
+
+        if command:
+            command_label = Gtk.Label(label=_("To add it, please run:"))
+            command_label.set_xalign(0)
+            box.append(command_label)
+
+            command_box = Gtk.Box(spacing=6)
+            command_box.set_hexpand(True)
+
+            command_entry = Gtk.Entry()
+            command_entry.set_editable(False)
+            command_entry.set_hexpand(True)
+            command_entry.set_text(command)
+            command_entry.add_css_class("monospace")
+            command_entry.set_width_chars(len(command))
+            command_entry.set_focusable(False)
+
+            btn_copy_command = Gtk.Button()
+            btn_copy_command.set_icon_name("edit-copy-symbolic")
+            btn_copy_command.set_tooltip_text(_("Copy command"))
+            btn_copy_command.connect(
+                "clicked", self.__copy_command_to_clipboard, command
+            )
+
+            command_box.append(command_entry)
+            command_box.append(btn_copy_command)
+            box.append(command_box)
+
+        popover.set_child(box)
+        return popover
+
+    def __add_unavailable_indicator(self, row: Adw.ActionRow, command: str | None):
+        if not row:
+            return
+
+        popover = self.__create_unavailable_popover(command)
+        menu_button = Gtk.MenuButton()
+        menu_button.set_valign(Gtk.Align.CENTER)
+        menu_button.set_icon_name("dialog-warning-symbolic")
+        menu_button.set_has_frame(False)
+        menu_button.set_popover(popover)
+        menu_button.set_tooltip_text(_("This feature is unavailable on your system."))
+
+        row.add_suffix(menu_button)
+
+    def __copy_command_to_clipboard(self, _widget, command: str):
+        display = Gdk.Display.get_default()
+        if not display:
+            return
+
+        clipboard = Gdk.Display.get_clipboard(display)
+        clipboard.set_content(Gdk.ContentProvider.new_for_value(command))
+        self.window.show_toast(_("Copied to clipboard"))
+
     def __check_entry_name(self, *_args):
         if self.entry_name.get_text() != self.config.Name:
             is_duplicate = self.entry_name.get_text() in self.manager.local_bottles
@@ -353,8 +434,7 @@ class PreferencesView(Adw.PreferencesPage):
             self.manager.update_config(
                 config=self.config, key="WorkingDir", value=dialog.get_file().get_path()
             )
-            self.label_cwd.set_label(os.path.basename(path))
-            self.btn_cwd_reset.set_visible(True)
+            self.__update_working_directory_row(path)
 
         dialog = Gtk.FileChooserNative.new(
             title=_("Select Working Directory"),
@@ -368,8 +448,23 @@ class PreferencesView(Adw.PreferencesPage):
 
     def reset_cwd(self, *_args):
         self.manager.update_config(config=self.config, key="WorkingDir", value="")
-        self.label_cwd.set_label("(Default)")
-        self.btn_cwd_reset.set_visible(False)
+        self.__update_working_directory_row()
+
+    def __update_working_directory_row(self, working_dir=None):
+        """Update the working directory."""
+
+        working_dir = working_dir if working_dir is not None else self.config.WorkingDir
+        has_custom_dir = bool(working_dir)
+
+        if has_custom_dir:
+            basename = os.path.basename(os.path.normpath(working_dir)) or working_dir
+            self.label_cwd.set_label(basename)
+            self.label_cwd.set_tooltip_text(working_dir)
+        else:
+            self.label_cwd.set_label(_("(Default)"))
+            self.label_cwd.set_tooltip_text(None)
+
+        self.btn_cwd_reset.set_visible(has_custom_dir)
 
     def update_combo_components(self):
         """
@@ -434,6 +529,8 @@ class PreferencesView(Adw.PreferencesPage):
         self.switch_mangohud.handler_block_by_func(self.__toggle_feature_cb)
         self.switch_nvapi.handler_block_by_func(self.__toggle_nvapi)
         self.switch_vkbasalt.handler_block_by_func(self.__toggle_feature_cb)
+        self.switch_wayland.handler_block_by_func(self.__toggle_wayland)
+        self.switch_winebridge.handler_block_by_func(self.__toggle_feature_cb)
         self.switch_fsr.handler_block_by_func(self.__toggle_feature_cb)
         self.switch_obsvkc.handler_block_by_func(self.__toggle_feature_cb)
         self.switch_gamemode.handler_block_by_func(self.__toggle_feature_cb)
@@ -457,6 +554,8 @@ class PreferencesView(Adw.PreferencesPage):
         self.switch_mangohud.set_active(parameters.mangohud)
         self.switch_obsvkc.set_active(parameters.obsvkc)
         self.switch_vkbasalt.set_active(parameters.vkbasalt)
+        self.switch_wayland.set_active(parameters.wayland)
+        self.switch_winebridge.set_active(parameters.winebridge)
         self.switch_fsr.set_active(parameters.fsr)
         self.switch_nvapi.set_active(parameters.dxvk_nvapi)
         self.switch_gamemode.set_active(parameters.gamemode)
@@ -476,7 +575,7 @@ class PreferencesView(Adw.PreferencesPage):
 
         self.switch_discrete.set_active(parameters.discrete_gpu)
 
-        self.btn_cwd_reset.set_visible(self.config.WorkingDir)
+        self.__update_working_directory_row()
 
         self.entry_name.set_text(config.Name)
 
@@ -568,6 +667,8 @@ class PreferencesView(Adw.PreferencesPage):
         self.switch_mangohud.handler_unblock_by_func(self.__toggle_feature_cb)
         self.switch_nvapi.handler_unblock_by_func(self.__toggle_nvapi)
         self.switch_vkbasalt.handler_unblock_by_func(self.__toggle_feature_cb)
+        self.switch_wayland.handler_unblock_by_func(self.__toggle_wayland)
+        self.switch_winebridge.handler_unblock_by_func(self.__toggle_feature_cb)
         self.switch_fsr.handler_unblock_by_func(self.__toggle_feature_cb)
         self.switch_obsvkc.handler_unblock_by_func(self.__toggle_feature_cb)
         self.switch_gamemode.handler_unblock_by_func(self.__toggle_feature_cb)
@@ -618,6 +719,11 @@ class PreferencesView(Adw.PreferencesPage):
     def __toggle_feature_cb(self, _widget: Gtk.Widget, state: bool, key: str) -> None:
         self.__toggle_feature(state=state, key=key)
 
+    def __toggle_wayland(self, _widget: Gtk.Widget, state: bool) -> None:
+        self.__toggle_feature(state=state, key="wayland")
+        rk = RegKeys(self.config)
+        RunAsync(rk.toggle_wayland_driver, state=state)
+
     def __set_sync_type(self, *_args):
         """
         Set the sync type (wine, esync, fsync)
